"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Graph = void 0;
class Graph {
    constructor(size) {
        this.size = size;
        this.matrix = Array.from({ length: size })
            .map(() => Array.from({ length: size })
            .map(() => Infinity));
    }
    setWeight(from, to, weight) {
        if (from >= this.size || from < 0 || to >= this.size || to < 0) {
            throw 'Out of bounds';
        }
        this.matrix[from][to] = weight;
        this.matrix[to][from] = weight;
    }
    get weights() {
        return JSON.parse(JSON.stringify(this.matrix));
    }
    getDistances(from) {
        if (from >= this.size || from < 0) {
            throw 'Out of bounds';
        }
        const distances = Array.from({ length: this.size })
            .map((_, i) => i === from ? 0 : Infinity);
        const unvisited = Array.from({ length: this.size })
            .map((_, i) => i);
        const visited = new Set();
        // while unvisited not empty
        while (unvisited.length > 0) {
            // pop from unvisited the node with min dist in distances and add it to visited
            unvisited.sort((nodeA, nodeB) => distances[nodeA] - distances[nodeB]);
            const currFromNode = unvisited.shift();
            visited.add(currFromNode);
            // for each node adjacent to the current one
            this.matrix[currFromNode].forEach((weight, toNode) => {
                // if (dist from source to current) + adjacent weight < distances[adjacent weight]
                const distance = distances[currFromNode] + this.matrix[currFromNode][toNode];
                if (distance < distances[toNode]) {
                    // replace distances[adjacent weight] with (dist from source to current) + adjacent weight
                    distances[toNode] = distance;
                }
            });
        }
        return distances;
    }
    getPath(from, to) {
        if (from >= this.size || from < 0 || to >= this.size || to < 0) {
            throw 'Out of bounds';
        }
        const path = [to];
        const distances = this.getDistances(from);
        let currentNode = to;
        while (currentNode !== from) {
            const adjacentNodes = Array.from({ length: this.size })
                .map((_, i) => i)
                .filter(node => this.matrix[currentNode][node] !== Infinity);
            if (adjacentNodes.length === 0) {
                return [];
            }
            const shortestAdjacentNode = adjacentNodes
                .sort((nodeA, nodeB) => (distances[nodeA] + this.matrix[currentNode][nodeA]) - (distances[nodeB] + this.matrix[currentNode][nodeB]))[0];
            if (distances[shortestAdjacentNode] === Infinity) {
                return [];
            }
            currentNode = shortestAdjacentNode;
            path.push(shortestAdjacentNode);
        }
        path.reverse();
        return path;
    }
}
exports.Graph = Graph;
//# sourceMappingURL=Graph.js.map